@@ -1,23 +1,27 @@
 """Array-agnostic implementations for the public API."""
 
-# https://github.com/scikit-learn/scikit-learn/pull/27910#issuecomment-2568023972
-from __future__ import annotations
-
 import math
-import operator
 import warnings
-from collections.abc import Callable
-from enum import Enum
-from types import ModuleType
-from typing import ClassVar, cast
+from collections.abc import Callable, Sequence
+from types import ModuleType, NoneType
+from typing import Literal, cast, overload
 
+from ._at import at
 from ._utils import _compat, _helpers
-from ._utils._compat import array_namespace, is_jax_array, is_writeable_array
-from ._utils._typing import Array, Index
+from ._utils._compat import array_namespace, is_dask_namespace, is_jax_array
+from ._utils._helpers import (
+    asarrays,
+    capabilities,
+    eager_shape,
+    meta_namespace,
+    ndindex,
+)
+from ._utils._typing import Array, Device, DType
 
 __all__ = [
-    "at",
+    "apply_where",
     "atleast_nd",
+    "broadcast_shapes",
     "cov",
     "create_diagonal",
     "expand_dims",
@@ -29,6 +33,148 @@ __all__ = [
 ]
 
 
+@overload
+def apply_where(  # numpydoc ignore=GL08
+    cond: Array,
+    args: Array | tuple[Array, ...],
+    f1: Callable[..., Array],
+    f2: Callable[..., Array],
+    /,
+    *,
+    xp: ModuleType | None = None,
+) -> Array: ...
+
+
+@overload
+def apply_where(  # numpydoc ignore=GL08
+    cond: Array,
+    args: Array | tuple[Array, ...],
+    f1: Callable[..., Array],
+    /,
+    *,
+    fill_value: Array | complex,
+    xp: ModuleType | None = None,
+) -> Array: ...
+
+
+def apply_where(  # numpydoc ignore=PR01,PR02
+    cond: Array,
+    args: Array | tuple[Array, ...],
+    f1: Callable[..., Array],
+    f2: Callable[..., Array] | None = None,
+    /,
+    *,
+    fill_value: Array | complex | None = None,
+    xp: ModuleType | None = None,
+) -> Array:
+    """
+    Run one of two elementwise functions depending on a condition.
+
+    Equivalent to ``f1(*args) if cond else fill_value`` performed elementwise
+    when `fill_value` is defined, otherwise to ``f1(*args) if cond else f2(*args)``.
+
+    Parameters
+    ----------
+    cond : array
+        The condition, expressed as a boolean array.
+    args : Array or tuple of Arrays
+        Argument(s) to `f1` (and `f2`). Must be broadcastable with `cond`.
+    f1 : callable
+        Elementwise function of `args`, returning a single array.
+        Where `cond` is True, output will be ``f1(arg0[cond], arg1[cond], ...)``.
+    f2 : callable, optional
+        Elementwise function of `args`, returning a single array.
+        Where `cond` is False, output will be ``f2(arg0[cond], arg1[cond], ...)``.
+        Mutually exclusive with `fill_value`.
+    fill_value : Array or scalar, optional
+        If provided, value with which to fill output array where `cond` is False.
+        It does not need to be scalar; it needs however to be broadcastable with
+        `cond` and `args`.
+        Mutually exclusive with `f2`. You must provide one or the other.
+    xp : array_namespace, optional
+        The standard-compatible namespace for `cond` and `args`. Default: infer.
+
+    Returns
+    -------
+    Array
+        An array with elements from the output of `f1` where `cond` is True and either
+        the output of `f2` or `fill_value` where `cond` is False. The returned array has
+        data type determined by type promotion rules between the output of `f1` and
+        either `fill_value` or the output of `f2`.
+
+    Notes
+    -----
+    ``xp.where(cond, f1(*args), f2(*args))`` requires explicitly evaluating `f1` even
+    when `cond` is False, and `f2` when cond is True. This function evaluates each
+    function only for their matching condition, if the backend allows for it.
+
+    On Dask, `f1` and `f2` are applied to the individual chunks and should use functions
+    from the namespace of the chunks.
+
+    Examples
+    --------
+    >>> import array_api_strict as xp
+    >>> import array_api_extra as xpx
+    >>> a = xp.asarray([5, 4, 3])
+    >>> b = xp.asarray([0, 2, 2])
+    >>> def f(a, b):
+    ...     return a // b
+    >>> xpx.apply_where(b != 0, (a, b), f, fill_value=xp.nan)
+    array([ nan,  2., 1.])
+    """
+    # Parse and normalize arguments
+    if (f2 is None) == (fill_value is None):
+        msg = "Exactly one of `fill_value` or `f2` must be given."
+        raise TypeError(msg)
+    args_ = list(args) if isinstance(args, tuple) else [args]
+    del args
+
+    xp = array_namespace(cond, fill_value, *args_) if xp is None else xp
+
+    if isinstance(fill_value, int | float | complex | NoneType):
+        cond, *args_ = xp.broadcast_arrays(cond, *args_)
+    else:
+        cond, fill_value, *args_ = xp.broadcast_arrays(cond, fill_value, *args_)
+
+    if is_dask_namespace(xp):
+        meta_xp = meta_namespace(cond, fill_value, *args_, xp=xp)
+        # map_blocks doesn't descend into tuples of Arrays
+        return xp.map_blocks(_apply_where, cond, f1, f2, fill_value, *args_, xp=meta_xp)
+    return _apply_where(cond, f1, f2, fill_value, *args_, xp=xp)
+
+
+def _apply_where(  # numpydoc ignore=PR01,RT01
+    cond: Array,
+    f1: Callable[..., Array],
+    f2: Callable[..., Array] | None,
+    fill_value: Array | int | float | complex | bool | None,
+    *args: Array,
+    xp: ModuleType,
+) -> Array:
+    """Helper of `apply_where`. On Dask, this runs on a single chunk."""
+
+    if not capabilities(xp, device=_compat.device(cond))["boolean indexing"]:
+        # jax.jit does not support assignment by boolean mask
+        return xp.where(cond, f1(*args), f2(*args) if f2 is not None else fill_value)
+
+    temp1 = f1(*(arr[cond] for arr in args))
+
+    if f2 is None:
+        dtype = xp.result_type(temp1, fill_value)
+        if isinstance(fill_value, int | float | complex):
+            out = xp.full_like(cond, dtype=dtype, fill_value=fill_value)
+        else:
+            out = xp.astype(fill_value, dtype, copy=True)
+    else:
+        ncond = ~cond
+        temp2 = f2(*(arr[ncond] for arr in args))
+        dtype = xp.result_type(temp1, temp2)
+        out = xp.empty_like(cond, dtype=dtype)
+        out = at(out, ncond).set(temp2)
+
+    return at(out, cond).set(temp1)
+
+
 def atleast_nd(x: Array, /, *, ndim: int, xp: ModuleType | None = None) -> Array:
     """
     Recursively expand the dimension of an array to at least `ndim`.
@@ -72,6 +218,69 @@ def atleast_nd(x: Array, /, *, ndim: int, xp: ModuleType | None = None) -> Array
     return x
 
 
+# `float` in signature to accept `math.nan` for Dask.
+# `int`s are still accepted as `float` is a superclass of `int` in typing
+def broadcast_shapes(*shapes: tuple[float | None, ...]) -> tuple[int | None, ...]:
+    """
+    Compute the shape of the broadcasted arrays.
+
+    Duplicates :func:`numpy.broadcast_shapes`, with additional support for
+    None and NaN sizes.
+
+    This is equivalent to ``xp.broadcast_arrays(arr1, arr2, ...)[0].shape``
+    without needing to worry about the backend potentially deep copying
+    the arrays.
+
+    Parameters
+    ----------
+    *shapes : tuple[int | None, ...]
+        Shapes of the arrays to broadcast.
+
+    Returns
+    -------
+    tuple[int | None, ...]
+        The shape of the broadcasted arrays.
+
+    See Also
+    --------
+    numpy.broadcast_shapes : Equivalent NumPy function.
+    array_api.broadcast_arrays : Function to broadcast actual arrays.
+
+    Notes
+    -----
+    This function accepts the Array API's ``None`` for unknown sizes,
+    as well as Dask's non-standard ``math.nan``.
+    Regardless of input, the output always contains ``None`` for unknown sizes.
+
+    Examples
+    --------
+    >>> import array_api_extra as xpx
+    >>> xpx.broadcast_shapes((2, 3), (2, 1))
+    (2, 3)
+    >>> xpx.broadcast_shapes((4, 2, 3), (2, 1), (1, 3))
+    (4, 2, 3)
+    """
+    if not shapes:
+        return ()  # Match NumPy output
+
+    ndim = max(len(shape) for shape in shapes)
+    out: list[int | None] = []
+    for axis in range(-ndim, 0):
+        sizes = {shape[axis] for shape in shapes if axis >= -len(shape)}
+        # Dask uses NaN for unknown shape, which predates the Array API spec for None
+        none_size = None in sizes or math.nan in sizes  # noqa: PLW0177
+        sizes -= {1, None, math.nan}
+        if len(sizes) > 1:
+            msg = (
+                "shape mismatch: objects cannot be broadcast to a single shape: "
+                f"{shapes}."
+            )
+            raise ValueError(msg)
+        out.append(None if none_size else cast(int, sizes.pop()) if sizes else 1)
+
+    return tuple(out)
+
+
 def cov(m: Array, /, *, xp: ModuleType | None = None) -> Array:
     """
     Estimate a covariance matrix.
@@ -148,11 +357,13 @@ def cov(m: Array, /, *, xp: ModuleType | None = None) -> Array:
     m = xp.astype(m, dtype)
 
     avg = _helpers.mean(m, axis=1, xp=xp)
-    fact = m.shape[1] - 1
+
+    m_shape = eager_shape(m)
+    fact = m_shape[1] - 1
 
     if fact <= 0:
         warnings.warn("Degrees of freedom <= 0 for slice", RuntimeWarning, stacklevel=2)
-        fact = 0.0
+        fact = 0
 
     m -= avg[:, None]
     m_transpose = m.T
@@ -164,6 +375,23 @@ def cov(m: Array, /, *, xp: ModuleType | None = None) -> Array:
     return xp.squeeze(c, axis=axes)
 
 
+def one_hot(
+    x: Array,
+    /,
+    num_classes: int,
+    *,
+    xp: ModuleType,
+) -> Array:  # numpydoc ignore=PR01,RT01
+    """See docstring in `array_api_extra._delegation.py`."""
+    # TODO: Benchmark whether this is faster on the NumPy backend:
+    # if is_numpy_array(x):
+    #     out = xp.zeros((x.size, num_classes), dtype=dtype)
+    #     out[xp.arange(x.size), xp.reshape(x, (-1,))] = 1
+    #     return xp.reshape(out, (*x.shape, num_classes))
+    range_num_classes = xp.arange(num_classes, dtype=x.dtype, device=_compat.device(x))
+    return x[..., xp.newaxis] == range_num_classes
+
+
 def create_diagonal(
     x: Array, /, *, offset: int = 0, xp: ModuleType | None = None
 ) -> Array:
@@ -173,7 +401,7 @@ def create_diagonal(
     Parameters
     ----------
     x : array
-        A 1-D array.
+        An array having shape ``(*batch_dims, k)``.
     offset : int, optional
         Offset from the leading diagonal (default is ``0``).
         Use positive ints for diagonals above the leading diagonal,
@@ -184,7 +412,8 @@ def create_diagonal(
     Returns
     -------
     array
-        A 2-D array with `x` on the diagonal (offset by `offset`).
+        An array having shape ``(*batch_dims, k+abs(offset), k+abs(offset))`` with `x`
+        on the diagonal (offset by `offset`).
 
     Examples
     --------
@@ -207,18 +436,61 @@ def create_diagonal(
     if xp is None:
         xp = array_namespace(x)
 
-    if x.ndim != 1:
-        err_msg = "`x` must be 1-dimensional."
+    if x.ndim == 0:
+        err_msg = "`x` must be at least 1-dimensional."
         raise ValueError(err_msg)
-    n = x.shape[0] + abs(offset)
-    diag = xp.zeros(n**2, dtype=x.dtype, device=_compat.device(x))
 
-    start = offset if offset >= 0 else abs(offset) * n
-    stop = min(n * (n - offset), diag.shape[0])
-    step = n + 1
-    diag = at(diag)[start:stop:step].set(x)
+    x_shape = eager_shape(x)
+    batch_dims = x_shape[:-1]
+    n = x_shape[-1] + abs(offset)
+    diag = xp.zeros((*batch_dims, n**2), dtype=x.dtype, device=_compat.device(x))
 
-    return xp.reshape(diag, (n, n))
+    target_slice = slice(
+        offset if offset >= 0 else abs(offset) * n,
+        min(n * (n - offset), diag.shape[-1]),
+        n + 1,
+    )
+    for index in ndindex(*batch_dims):
+        diag = at(diag)[(*index, target_slice)].set(x[(*index, slice(None))])
+    return xp.reshape(diag, (*batch_dims, n, n))
+
+
+def default_dtype(
+    xp: ModuleType,
+    kind: Literal[
+        "real floating", "complex floating", "integral", "indexing"
+    ] = "real floating",
+    *,
+    device: Device | None = None,
+) -> DType:
+    """
+    Return the default dtype for the given namespace and device.
+
+    This is a convenience shorthand for
+    ``xp.__array_namespace_info__().default_dtypes(device=device)[kind]``.
+
+    Parameters
+    ----------
+    xp : array_namespace
+        The standard-compatible namespace for which to get the default dtype.
+    kind : {'real floating', 'complex floating', 'integral', 'indexing'}, optional
+        The kind of dtype to return. Default is 'real floating'.
+    device : Device, optional
+        The device for which to get the default dtype. Default: current device.
+
+    Returns
+    -------
+    dtype
+        The default dtype for the given namespace, kind, and device.
+    """
+    dtypes = xp.__array_namespace_info__().default_dtypes(device=device)
+    try:
+        return dtypes[kind]
+    except KeyError as e:
+        domain = ("real floating", "complex floating", "integral", "indexing")
+        assert set(dtypes) == set(domain), f"Non-compliant namespace: {dtypes}"
+        msg = f"Unknown kind '{kind}'. Expected one of {domain}."
+        raise ValueError(msg) from e
 
 
 def expand_dims(
@@ -307,7 +579,60 @@ def expand_dims(
     return a
 
 
-def kron(a: Array, b: Array, /, *, xp: ModuleType | None = None) -> Array:
+def isclose(
+    a: Array | complex,
+    b: Array | complex,
+    *,
+    rtol: float = 1e-05,
+    atol: float = 1e-08,
+    equal_nan: bool = False,
+    xp: ModuleType,
+) -> Array:  # numpydoc ignore=PR01,RT01
+    """See docstring in array_api_extra._delegation."""
+    a, b = asarrays(a, b, xp=xp)
+
+    a_inexact = xp.isdtype(a.dtype, ("real floating", "complex floating"))
+    b_inexact = xp.isdtype(b.dtype, ("real floating", "complex floating"))
+    if a_inexact or b_inexact:
+        # prevent warnings on NumPy and Dask on inf - inf
+        mxp = meta_namespace(a, b, xp=xp)
+        out = apply_where(
+            xp.isinf(a) | xp.isinf(b),
+            (a, b),
+            lambda a, b: mxp.isinf(a) & mxp.isinf(b) & (mxp.sign(a) == mxp.sign(b)),  # pyright: ignore[reportUnknownArgumentType]
+            # Note: inf <= inf is True!
+            lambda a, b: mxp.abs(a - b) <= (atol + rtol * mxp.abs(b)),  # pyright: ignore[reportUnknownArgumentType]
+            xp=xp,
+        )
+        if equal_nan:
+            out = xp.where(xp.isnan(a) & xp.isnan(b), True, out)
+        return out
+
+    if xp.isdtype(a.dtype, "bool") or xp.isdtype(b.dtype, "bool"):
+        if atol >= 1 or rtol >= 1:
+            return xp.ones_like(a == b)
+        return a == b
+
+    # integer types
+    atol = int(atol)
+    if rtol == 0:
+        return xp.abs(a - b) <= atol
+
+    # Don't rely on OverflowError, as it is not guaranteed by the Array API.
+    nrtol = int(1.0 / rtol)
+    if nrtol > xp.iinfo(b.dtype).max:
+        # rtol * max_int < 1, so it's inconsequential
+        return xp.abs(a - b) <= atol
+    return xp.abs(a - b) <= (atol + xp.abs(b) // nrtol)
+
+
+def kron(
+    a: Array | complex,
+    b: Array | complex,
+    /,
+    *,
+    xp: ModuleType | None = None,
+) -> Array:
     """
     Kronecker product of two arrays.
 
@@ -318,8 +643,8 @@ def kron(a: Array, b: Array, /, *, xp: ModuleType | None = None) -> Array:
 
     Parameters
     ----------
-    a, b : array
-        Input arrays.
+    a, b : Array | int | float | complex
+        Input arrays or scalars. At least one must be an array.
     xp : array_namespace, optional
         The standard-compatible namespace for `a` and `b`. Default: infer.
 
@@ -382,18 +707,18 @@ def kron(a: Array, b: Array, /, *, xp: ModuleType | None = None) -> Array:
     """
     if xp is None:
         xp = array_namespace(a, b)
+    a, b = asarrays(a, b, xp=xp)
 
-    b = xp.asarray(b)
     singletons = (1,) * (b.ndim - a.ndim)
-    a = xp.broadcast_to(xp.asarray(a), singletons + a.shape)
+    a = cast(Array, xp.broadcast_to(a, singletons + a.shape))
 
     nd_b, nd_a = b.ndim, a.ndim
     nd_max = max(nd_b, nd_a)
     if nd_a == 0 or nd_b == 0:
         return xp.multiply(a, b)
 
-    a_shape = a.shape
-    b_shape = b.shape
+    a_shape = eager_shape(a)
+    b_shape = eager_shape(b)
 
     # Equalise the shapes by prepending smaller one with 1s
     a_shape = (1,) * max(0, nd_b - nd_a) + a_shape
@@ -413,9 +738,115 @@ def kron(a: Array, b: Array, /, *, xp: ModuleType | None = None) -> Array:
     return xp.reshape(result, res_shape)
 
 
+def nunique(x: Array, /, *, xp: ModuleType | None = None) -> Array:
+    """
+    Count the number of unique elements in an array.
+
+    Compatible with JAX and Dask, whose laziness would be otherwise
+    problematic.
+
+    Parameters
+    ----------
+    x : Array
+        Input array.
+    xp : array_namespace, optional
+        The standard-compatible namespace for `x`. Default: infer.
+
+    Returns
+    -------
+    array: 0-dimensional integer array
+        The number of unique elements in `x`. It can be lazy.
+    """
+    if xp is None:
+        xp = array_namespace(x)
+
+    if is_jax_array(x):
+        # size= is JAX-specific
+        # https://github.com/data-apis/array-api/issues/883
+        _, counts = xp.unique_counts(x, size=_compat.size(x))
+        return (counts > 0).sum()
+
+    # There are 3 general use cases:
+    # 1. backend has unique_counts and it returns an array with known shape
+    # 2. backend has unique_counts and it returns a None-sized array;
+    #    e.g. Dask, ndonnx
+    # 3. backend does not have unique_counts; e.g. wrapped JAX
+    if capabilities(xp, device=_compat.device(x))["data-dependent shapes"]:
+        # xp has unique_counts; O(n) complexity
+        _, counts = xp.unique_counts(x)
+        n = _compat.size(counts)
+        if n is None:
+            return xp.sum(xp.ones_like(counts))
+        return xp.asarray(n, device=_compat.device(x))
+
+    # xp does not have unique_counts; O(n*logn) complexity
+    x = xp.reshape(x, (-1,))
+    x = xp.sort(x)
+    mask = x != xp.roll(x, -1)
+    default_int = default_dtype(xp, "integral", device=_compat.device(x))
+    return xp.maximum(
+        # Special cases:
+        # - array is size 0
+        # - array has all elements equal to each other
+        xp.astype(xp.any(~mask), default_int),
+        xp.sum(xp.astype(mask, default_int)),
+    )
+
+
+def pad(
+    x: Array,
+    pad_width: int | tuple[int, int] | Sequence[tuple[int, int]],
+    *,
+    constant_values: complex = 0,
+    xp: ModuleType,
+) -> Array:  # numpydoc ignore=PR01,RT01
+    """See docstring in `array_api_extra._delegation.py`."""
+    # make pad_width a list of length-2 tuples of ints
+    if isinstance(pad_width, int):
+        pad_width_seq = [(pad_width, pad_width)] * x.ndim
+    elif (
+        isinstance(pad_width, tuple)
+        and len(pad_width) == 2
+        and all(isinstance(i, int) for i in pad_width)
+    ):
+        pad_width_seq = [cast(tuple[int, int], pad_width)] * x.ndim
+    else:
+        pad_width_seq = cast(list[tuple[int, int]], list(pad_width))
+
+    slices: list[slice] = []
+    newshape: list[int] = []
+    for ax, w_tpl in enumerate(pad_width_seq):
+        if len(w_tpl) != 2:
+            msg = f"expect a 2-tuple (before, after), got {w_tpl}."
+            raise ValueError(msg)
+
+        sh = eager_shape(x)[ax]
+
+        if w_tpl[0] == 0 and w_tpl[1] == 0:
+            sl = slice(None, None, None)
+        else:
+            stop: int | None
+            start, stop = w_tpl
+            stop = None if stop == 0 else -stop
+
+            sl = slice(start, stop, None)
+            sh += w_tpl[0] + w_tpl[1]
+
+        newshape.append(sh)
+        slices.append(sl)
+
+    padded = xp.full(
+        tuple(newshape),
+        fill_value=constant_values,
+        dtype=x.dtype,
+        device=_compat.device(x),
+    )
+    return at(padded, tuple(slices)).set(x)
+
+
 def setdiff1d(
-    x1: Array,
-    x2: Array,
+    x1: Array | complex,
+    x2: Array | complex,
     /,
     *,
     assume_unique: bool = False,
@@ -428,7 +859,7 @@ def setdiff1d(
 
     Parameters
     ----------
-    x1 : array
+    x1 : array | int | float | complex | bool
         Input array.
     x2 : array
         Input comparison array.
@@ -457,13 +888,17 @@ def setdiff1d(
     """
     if xp is None:
         xp = array_namespace(x1, x2)
+    # https://github.com/microsoft/pyright/issues/10103
+    x1_, x2_ = asarrays(x1, x2, xp=xp)
 
     if assume_unique:
-        x1 = xp.reshape(x1, (-1,))
+        x1_ = xp.reshape(x1_, (-1,))
+        x2_ = xp.reshape(x2_, (-1,))
     else:
-        x1 = xp.unique_values(x1)
-        x2 = xp.unique_values(x2)
-    return x1[_helpers.in1d(x1, x2, assume_unique=True, invert=True, xp=xp)]
+        x1_ = xp.unique_values(x1_)
+        x2_ = xp.unique_values(x2_)
+
+    return x1_[_helpers.in1d(x1_, x2_, assume_unique=True, invert=True, xp=xp)]
 
 
 def sinc(x: Array, /, *, xp: ModuleType | None = None) -> Array:
@@ -553,444 +988,3 @@ def sinc(x: Array, /, *, xp: ModuleType | None = None) -> Array:
         xp.asarray(xp.finfo(x.dtype).eps, dtype=x.dtype, device=_compat.device(x)),
     )
     return xp.sin(y) / y
-
-
-def pad(
-    x: Array,
-    pad_width: int | tuple[int, int] | list[tuple[int, int]],
-    *,
-    constant_values: bool | int | float | complex = 0,
-    xp: ModuleType,
-) -> Array:  # numpydoc ignore=PR01,RT01
-    """See docstring in `array_api_extra._delegation.py`."""
-    # make pad_width a list of length-2 tuples of ints
-    x_ndim = cast(int, x.ndim)
-    if isinstance(pad_width, int):
-        pad_width = [(pad_width, pad_width)] * x_ndim
-    if isinstance(pad_width, tuple):
-        pad_width = [pad_width] * x_ndim
-
-    # https://github.com/python/typeshed/issues/13376
-    slices: list[slice] = []  # type: ignore[no-any-explicit]
-    newshape: list[int] = []
-    for ax, w_tpl in enumerate(pad_width):
-        if len(w_tpl) != 2:
-            msg = f"expect a 2-tuple (before, after), got {w_tpl}."
-            raise ValueError(msg)
-
-        sh = x.shape[ax]
-        if w_tpl[0] == 0 and w_tpl[1] == 0:
-            sl = slice(None, None, None)
-        else:
-            start, stop = w_tpl
-            stop = None if stop == 0 else -stop
-
-            sl = slice(start, stop, None)
-            sh += w_tpl[0] + w_tpl[1]
-
-        newshape.append(sh)
-        slices.append(sl)
-
-    padded = xp.full(
-        tuple(newshape),
-        fill_value=constant_values,
-        dtype=x.dtype,
-        device=_compat.device(x),
-    )
-    return at(padded, tuple(slices)).set(x)
-
-
-def nunique(x: Array, /, *, xp: ModuleType | None = None) -> Array:
-    """
-    Count the number of unique elements in an array.
-
-    Compatible with JAX and Dask, whose laziness would be otherwise
-    problematic.
-
-    Parameters
-    ----------
-    x : Array
-        Input array.
-    xp : array_namespace, optional
-        The standard-compatible namespace for `x`. Default: infer.
-
-    Returns
-    -------
-    array: 0-dimensional integer array
-        The number of unique elements in `x`. It can be lazy.
-    """
-    if xp is None:
-        xp = array_namespace(x)
-
-    if is_jax_array(x):
-        # size= is JAX-specific
-        # https://github.com/data-apis/array-api/issues/883
-        _, counts = xp.unique_counts(x, size=_compat.size(x))
-        return xp.astype(counts, xp.bool).sum()
-
-    _, counts = xp.unique_counts(x)
-    n = _compat.size(counts)
-    # FIXME https://github.com/data-apis/array-api-compat/pull/231
-    if n is None or math.isnan(n):  # e.g. Dask, ndonnx
-        return xp.astype(counts, xp.bool).sum()
-    return xp.asarray(n, device=_compat.device(x))
-
-
-class _AtOp(Enum):
-    """Operations for use in `xpx.at`."""
-
-    SET = "set"
-    ADD = "add"
-    SUBTRACT = "subtract"
-    MULTIPLY = "multiply"
-    DIVIDE = "divide"
-    POWER = "power"
-    MIN = "min"
-    MAX = "max"
-
-    # @override from Python 3.12
-    def __str__(self) -> str:  # type: ignore[explicit-override]  # pyright: ignore[reportImplicitOverride]
-        """
-        Return string representation (useful for pytest logs).
-
-        Returns
-        -------
-        str
-            The operation's name.
-        """
-        return self.value
-
-
-_undef = object()
-
-
-class at:  # pylint: disable=invalid-name  # numpydoc ignore=PR02
-    """
-    Update operations for read-only arrays.
-
-    This implements ``jax.numpy.ndarray.at`` for all writeable
-    backends (those that support ``__setitem__``) and routes
-    to the ``.at[]`` method for JAX arrays.
-
-    Parameters
-    ----------
-    x : array
-        Input array.
-    idx : index, optional
-        Only `array API standard compliant indices
-        <https://data-apis.org/array-api/latest/API_specification/indexing.html>`_
-        are supported.
-
-        You may use two alternate syntaxes::
-
-          >>> import array_api_extra as xpx
-          >>> xpx.at(x, idx).set(value)  # or add(value), etc.
-          >>> xpx.at(x)[idx].set(value)
-
-    copy : bool, optional
-        None (default)
-            The array parameter *may* be modified in place if it is
-            possible and beneficial for performance.
-            You should not reuse it after calling this function.
-        True
-            Ensure that the inputs are not modified.
-        False
-            Ensure that the update operation writes back to the input.
-            Raise ``ValueError`` if a copy cannot be avoided.
-
-    xp : array_namespace, optional
-        The standard-compatible namespace for `x`. Default: infer.
-
-    Returns
-    -------
-    Updated input array.
-
-    Warnings
-    --------
-    (a) When you omit the ``copy`` parameter, you should never reuse the parameter
-    array later on; ideally, you should reassign it immediately::
-
-        >>> import array_api_extra as xpx
-        >>> x = xpx.at(x, 0).set(2)
-
-    The above best practice pattern ensures that the behaviour won't change depending
-    on whether ``x`` is writeable or not, as the original ``x`` object is dereferenced
-    as soon as ``xpx.at`` returns; this way there is no risk to accidentally update it
-    twice.
-
-    On the reverse, the anti-pattern below must be avoided, as it will result in
-    different behaviour on read-only versus writeable arrays::
-
-        >>> x = xp.asarray([0, 0, 0])
-        >>> y = xpx.at(x, 0).set(2)
-        >>> z = xpx.at(x, 1).set(3)
-
-    In the above example, both calls to ``xpx.at`` update ``x`` in place *if possible*.
-    This causes the behaviour to diverge depending on whether ``x`` is writeable or not:
-
-    - If ``x`` is writeable, then after the snippet above you'll have
-      ``x == y == z == [2, 3, 0]``
-    - If ``x`` is read-only, then you'll end up with
-      ``x == [0, 0, 0]``, ``y == [2, 0, 0]`` and ``z == [0, 3, 0]``.
-
-    The correct pattern to use if you want diverging outputs from the same input is
-    to enforce copies::
-
-        >>> x = xp.asarray([0, 0, 0])
-        >>> y = xpx.at(x, 0).set(2, copy=True)  # Never updates x
-        >>> z = xpx.at(x, 1).set(3)  # May or may not update x in place
-        >>> del x  # avoid accidental reuse of x as we don't know its state anymore
-
-    (b) The array API standard does not support integer array indices.
-    The behaviour of update methods when the index is an array of integers is
-    undefined and will vary between backends; this is particularly true when the
-    index contains multiple occurrences of the same index, e.g.::
-
-        >>> import numpy as np
-        >>> import jax.numpy as jnp
-        >>> import array_api_extra as xpx
-        >>> xpx.at(np.asarray([123]), np.asarray([0, 0])).add(1)
-        array([124])
-        >>> xpx.at(jnp.asarray([123]), jnp.asarray([0, 0])).add(1)
-        Array([125], dtype=int32)
-
-    See Also
-    --------
-    jax.numpy.ndarray.at : Equivalent array method in JAX.
-
-    Notes
-    -----
-    `sparse <https://sparse.pydata.org/>`_, as well as read-only arrays from libraries
-    not explicitly covered by ``array-api-compat``, are not supported by update
-    methods.
-
-    Examples
-    --------
-    Given either of these equivalent expressions::
-
-      >>> import array_api_extra as xpx
-      >>> x = xpx.at(x)[1].add(2)
-      >>> x = xpx.at(x, 1).add(2)
-
-    If x is a JAX array, they are the same as::
-
-      >>> x = x.at[1].add(2)
-
-    If x is a read-only numpy array, they are the same as::
-
-      >>> x = x.copy()
-      >>> x[1] += 2
-
-    For other known backends, they are the same as::
-
-      >>> x[1] += 2
-    """
-
-    _x: Array
-    _idx: Index
-    __slots__: ClassVar[tuple[str, ...]] = ("_idx", "_x")
-
-    def __init__(
-        self, x: Array, idx: Index = _undef, /
-    ) -> None:  # numpydoc ignore=GL08
-        self._x = x
-        self._idx = idx
-
-    def __getitem__(self, idx: Index, /) -> at:  # numpydoc ignore=PR01,RT01
-        """
-        Allow for the alternate syntax ``at(x)[start:stop:step]``.
-
-        It looks prettier than ``at(x, slice(start, stop, step))``
-        and feels more intuitive coming from the JAX documentation.
-        """
-        if self._idx is not _undef:
-            msg = "Index has already been set"
-            raise ValueError(msg)
-        return at(self._x, idx)
-
-    def _update_common(
-        self,
-        at_op: _AtOp,
-        y: Array,
-        /,
-        copy: bool | None,
-        xp: ModuleType | None,
-    ) -> tuple[Array, None] | tuple[None, Array]:  # numpydoc ignore=PR01
-        """
-        Perform common prepocessing to all update operations.
-
-        Returns
-        -------
-        tuple
-            If the operation can be resolved by ``at[]``, ``(return value, None)``
-            Otherwise, ``(None, preprocessed x)``.
-        """
-        x, idx = self._x, self._idx
-
-        if idx is _undef:
-            msg = (
-                "Index has not been set.\n"
-                "Usage: either\n"
-                "    at(x, idx).set(value)\n"
-                "or\n"
-                "    at(x)[idx].set(value)\n"
-                "(same for all other methods)."
-            )
-            raise ValueError(msg)
-
-        if copy not in (True, False, None):
-            msg = f"copy must be True, False, or None; got {copy!r}"
-            raise ValueError(msg)
-
-        if copy is None:
-            writeable = is_writeable_array(x)
-            copy = not writeable
-        elif copy:
-            writeable = None
-        else:
-            writeable = is_writeable_array(x)
-
-        if copy:
-            if is_jax_array(x):
-                # Use JAX's at[]
-                func = cast(Callable[[Array], Array], getattr(x.at[idx], at_op.value))
-                return func(y), None
-            # Emulate at[] behaviour for non-JAX arrays
-            # with a copy followed by an update
-            if xp is None:
-                xp = array_namespace(x)
-            x = xp.asarray(x, copy=True)
-            if writeable is False:
-                # A copy of a read-only numpy array is writeable
-                # Note: this assumes that a copy of a writeable array is writeable
-                writeable = None
-
-        if writeable is None:
-            writeable = is_writeable_array(x)
-        if not writeable:
-            # sparse crashes here
-            msg = f"Can't update read-only array {x}"
-            raise ValueError(msg)
-
-        return None, x
-
-    def set(
-        self,
-        y: Array,
-        /,
-        copy: bool | None = None,
-        xp: ModuleType | None = None,
-    ) -> Array:  # numpydoc ignore=PR01,RT01
-        """Apply ``x[idx] = y`` and return the update array."""
-        res, x = self._update_common(_AtOp.SET, y, copy=copy, xp=xp)
-        if res is not None:
-            return res
-        assert x is not None
-        x[self._idx] = y
-        return x
-
-    def _iop(
-        self,
-        at_op: _AtOp,
-        elwise_op: Callable[[Array, Array], Array],
-        y: Array,
-        /,
-        copy: bool | None,
-        xp: ModuleType | None,
-    ) -> Array:  # numpydoc ignore=PR01,RT01
-        """
-        ``x[idx] += y`` or equivalent in-place operation on a subset of x.
-
-        which is the same as saying
-            x[idx] = x[idx] + y
-        Note that this is not the same as
-            operator.iadd(x[idx], y)
-        Consider for example when x is a numpy array and idx is a fancy index, which
-        triggers a deep copy on __getitem__.
-        """
-        res, x = self._update_common(at_op, y, copy=copy, xp=xp)
-        if res is not None:
-            return res
-        assert x is not None
-        x[self._idx] = elwise_op(x[self._idx], y)
-        return x
-
-    def add(
-        self,
-        y: Array,
-        /,
-        copy: bool | None = None,
-        xp: ModuleType | None = None,
-    ) -> Array:  # numpydoc ignore=PR01,RT01
-        """Apply ``x[idx] += y`` and return the updated array."""
-
-        # Note for this and all other methods based on _iop:
-        # operator.iadd and operator.add subtly differ in behaviour, as
-        # only iadd will trigger exceptions when y has an incompatible dtype.
-        return self._iop(_AtOp.ADD, operator.iadd, y, copy=copy, xp=xp)
-
-    def subtract(
-        self,
-        y: Array,
-        /,
-        copy: bool | None = None,
-        xp: ModuleType | None = None,
-    ) -> Array:  # numpydoc ignore=PR01,RT01
-        """Apply ``x[idx] -= y`` and return the updated array."""
-        return self._iop(_AtOp.SUBTRACT, operator.isub, y, copy=copy, xp=xp)
-
-    def multiply(
-        self,
-        y: Array,
-        /,
-        copy: bool | None = None,
-        xp: ModuleType | None = None,
-    ) -> Array:  # numpydoc ignore=PR01,RT01
-        """Apply ``x[idx] *= y`` and return the updated array."""
-        return self._iop(_AtOp.MULTIPLY, operator.imul, y, copy=copy, xp=xp)
-
-    def divide(
-        self,
-        y: Array,
-        /,
-        copy: bool | None = None,
-        xp: ModuleType | None = None,
-    ) -> Array:  # numpydoc ignore=PR01,RT01
-        """Apply ``x[idx] /= y`` and return the updated array."""
-        return self._iop(_AtOp.DIVIDE, operator.itruediv, y, copy=copy, xp=xp)
-
-    def power(
-        self,
-        y: Array,
-        /,
-        copy: bool | None = None,
-        xp: ModuleType | None = None,
-    ) -> Array:  # numpydoc ignore=PR01,RT01
-        """Apply ``x[idx] **= y`` and return the updated array."""
-        return self._iop(_AtOp.POWER, operator.ipow, y, copy=copy, xp=xp)
-
-    def min(
-        self,
-        y: Array,
-        /,
-        copy: bool | None = None,
-        xp: ModuleType | None = None,
-    ) -> Array:  # numpydoc ignore=PR01,RT01
-        """Apply ``x[idx] = minimum(x[idx], y)`` and return the updated array."""
-        if xp is None:
-            xp = array_namespace(self._x)
-        y = xp.asarray(y)
-        return self._iop(_AtOp.MIN, xp.minimum, y, copy=copy, xp=xp)
-
-    def max(
-        self,
-        y: Array,
-        /,
-        copy: bool | None = None,
-        xp: ModuleType | None = None,
-    ) -> Array:  # numpydoc ignore=PR01,RT01
-        """Apply ``x[idx] = maximum(x[idx], y)`` and return the updated array."""
-        if xp is None:
-            xp = array_namespace(self._x)
-        y = xp.asarray(y)
-        return self._iop(_AtOp.MAX, xp.maximum, y, copy=copy, xp=xp)
