@@ -27,18 +27,13 @@ from array_api_extra import (
     setdiff1d,
     sinc,
 )
-from array_api_extra._lib._backends import Backend
+from array_api_extra._lib._backends import NUMPY_VERSION, Backend
 from array_api_extra._lib._testing import xp_assert_close, xp_assert_equal
 from array_api_extra._lib._utils._compat import device as get_device
 from array_api_extra._lib._utils._helpers import eager_shape, ndindex
 from array_api_extra._lib._utils._typing import Array, Device
 from array_api_extra.testing import lazy_xp_function
 
-from .conftest import NUMPY_VERSION
-
-# some xp backends are untyped
-# mypy: disable-error-code=no-untyped-def
-
 lazy_xp_function(apply_where)
 lazy_xp_function(atleast_nd)
 lazy_xp_function(cov)
@@ -99,8 +94,8 @@ class TestApplyWhere:
         actual = apply_where(
             cond,
             (x, y),
-            lambda x, _: x,  # pyright: ignore[reportUnknownArgumentType]
-            lambda _, y: y,  # pyright: ignore[reportUnknownArgumentType]
+            lambda x, _: x,
+            lambda _, y: y,
         )
         expect = xp.where(cond, x, y)
         xp_assert_equal(actual, expect)
@@ -120,7 +115,7 @@ class TestApplyWhere:
             cond,
             (x, y),
             self.f1,
-            lambda x, y: mxp.astype(x - y, xp.int64),  # pyright: ignore[reportArgumentType,reportUnknownArgumentType]
+            lambda x, y: mxp.astype(x - y, xp.int64),  # pyright: ignore[reportArgumentType]
         )
         assert actual.dtype == xp.int64
 
@@ -167,8 +162,8 @@ class TestApplyWhere:
         actual = apply_where(
             x == 0,
             (x, y),
-            lambda x, y: x / y,  # pyright: ignore[reportUnknownArgumentType]
-            lambda x, y: y / x,  # pyright: ignore[reportUnknownArgumentType]
+            lambda x, y: x / y,
+            lambda x, y: y / x,
         )
         xp_assert_equal(actual, xp.asarray([0.0, 1.5, 0.0]))
 
@@ -213,7 +208,7 @@ class TestApplyWhere:
         p=st.floats(min_value=0, max_value=1),
         data=st.data(),
     )
-    def test_hypothesis(  # type: ignore[explicit-any,decorated-any]
+    def test_hypothesis(
         self,
         n_arrays: int,
         rng_seed: int,
@@ -416,7 +411,7 @@ class TestCov:
         expect = xp.asarray([[1.0, -1.0j], [1.0j, 1.0]], dtype=xp.complex128)
         xp_assert_close(actual, expect)
 
-    @pytest.mark.skip_xp_backend(Backend.SPARSE, reason="matmul with nan fillvalue")
+    @pytest.mark.xfail_xp_backend(Backend.SPARSE, reason="sparse#877")
     def test_empty(self, xp: ModuleType):
         with warnings.catch_warnings(record=True):
             warnings.simplefilter("always", RuntimeWarning)
@@ -451,7 +446,7 @@ class TestCov:
         )
 
 
-@pytest.mark.skip_xp_backend(Backend.SPARSE, reason="backend doesn't have arange")
+@pytest.mark.xfail_xp_backend(Backend.SPARSE, reason="no arange", strict=False)
 class TestOneHot:
     @pytest.mark.parametrize("n_dim", range(4))
     @pytest.mark.parametrize("num_classes", [1, 3, 10])
@@ -816,7 +811,7 @@ class TestIsClose:
             isclose(xp.asarray(True), b, atol=1), xp.asarray([True, True, True])
         )
 
-    @pytest.mark.xfail_xp_backend(Backend.SPARSE, reason="index by sparse array")
+    @pytest.mark.skip_xp_backend(Backend.SPARSE, reason="index by sparse array")
     @pytest.mark.skip_xp_backend(Backend.ARRAY_API_STRICTEST, reason="unknown shape")
     def test_none_shape(self, xp: ModuleType):
         a = xp.asarray([1, 5, 0])
@@ -825,7 +820,7 @@ class TestIsClose:
         a = a[a < 5]
         xp_assert_equal(isclose(a, b), xp.asarray([True, False]))
 
-    @pytest.mark.xfail_xp_backend(Backend.SPARSE, reason="index by sparse array")
+    @pytest.mark.skip_xp_backend(Backend.SPARSE, reason="index by sparse array")
     @pytest.mark.skip_xp_backend(Backend.ARRAY_API_STRICTEST, reason="unknown shape")
     def test_none_shape_bool(self, xp: ModuleType):
         a = xp.asarray([True, True, False])
@@ -1141,10 +1136,10 @@ class TestSetDiff1D:
 
 
 class TestSinc:
-    @pytest.mark.xfail_xp_backend(Backend.SPARSE, reason="no linspace")
     def test_simple(self, xp: ModuleType):
         xp_assert_equal(sinc(xp.asarray(0.0)), xp.asarray(1.0))
-        w = sinc(xp.linspace(-1, 1, 100))
+        x = xp.asarray(np.linspace(-1, 1, 100))
+        w = sinc(x)
         # check symmetry
         xp_assert_close(w, xp.flip(w, axis=0))
 
@@ -1153,11 +1148,12 @@ class TestSinc:
         with pytest.raises(ValueError, match="real floating data type"):
             _ = sinc(xp.asarray(x))
 
-    @pytest.mark.xfail_xp_backend(Backend.SPARSE, reason="no arange")
     def test_3d(self, xp: ModuleType):
-        x = xp.reshape(xp.arange(18, dtype=xp.float64), (3, 3, 2))
-        expected = xp.zeros((3, 3, 2), dtype=xp.float64)
-        expected = at(expected)[0, 0, 0].set(1.0)
+        x = np.arange(18, dtype=np.float64).reshape((3, 3, 2))
+        expected = np.zeros_like(x)
+        expected[0, 0, 0] = 1
+        x = xp.asarray(x)
+        expected = xp.asarray(expected)
         xp_assert_close(sinc(x), expected, atol=1e-15)
 
     def test_device(self, xp: ModuleType, device: Device):
