@@ -18,9 +18,8 @@ from array_api_extra._lib._utils._compat import (
     is_jax_namespace,
 )
 from array_api_extra._lib._utils._typing import Array, Device
-from array_api_extra.testing import lazy_xp_function
+from array_api_extra.testing import lazy_xp_function, patch_lazy_xp_functions
 
-# mypy: disable-error-code="decorated-any, explicit-any"
 # pyright: reportUnknownParameterType=false,reportMissingParameterType=false
 
 
@@ -140,7 +139,7 @@ class TestAssertEqualCloseLess:
             xp_assert_less(xp.asarray([1, 1]), xp.asarray([2, 1]))
 
     @pytest.mark.parametrize("func", [xp_assert_equal, xp_assert_close, xp_assert_less])
-    @pytest.mark.xfail_xp_backend(Backend.SPARSE, reason="index by sparse array")
+    @pytest.mark.skip_xp_backend(Backend.SPARSE, reason="index by sparse array")
     @pytest.mark.skip_xp_backend(Backend.ARRAY_API_STRICTEST, reason="boolean indexing")
     def test_none_shape(self, xp: ModuleType, func: Callable[..., None]):
         """On Dask and other lazy backends, test that a shape with NaN's or None's
@@ -295,9 +294,9 @@ try:
     # Test an arbitrary Cython ufunc (@cython.vectorize).
     # When SCIPY_ARRAY_API is not set, this is the same as
     # scipy.special.erf.
-    from scipy.special._ufuncs import erf  # type: ignore[import-not-found]
+    from scipy.special._ufuncs import erf  # type: ignore[import-untyped]
 
-    lazy_xp_function(erf)  # pyright: ignore[reportUnknownArgumentType]
+    lazy_xp_function(erf)
 except ImportError:
     erf = None
 
@@ -449,3 +448,23 @@ def test_lazy_xp_modules(xp: ModuleType, library: Backend):
     else:
         y = wrapped.f(x)
         xp_assert_equal(y, x)
+
+
+def test_patch_lazy_xp_functions_deprecated_monkeypatch(
+    request: pytest.FixtureRequest, monkeypatch: pytest.MonkeyPatch
+):
+    xp = pytest.importorskip("dask.array")
+
+    x = xp.asarray([1.0, 2.0])
+    y = non_materializable5(x)
+    xp_assert_equal(y, x)
+
+    with pytest.warns(DeprecationWarning, match="`monkeypatch` parameter"):
+        _ = patch_lazy_xp_functions(request, monkeypatch, xp=xp)
+
+    with pytest.raises(AssertionError, match=r"dask\.compute.* 1 times"):
+        _ = non_materializable5(x)
+
+    monkeypatch.undo()
+    y = non_materializable5(x)
+    xp_assert_equal(y, x)
